"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var apollo_link_1 = require("apollo-link");
var apollo_utilities_1 = require("apollo-utilities");
var graphql_1 = require("graphql");
var graphql_utils_1 = require("./graphql-utils");
function createFirestoreLink(_a) {
    var database = _a.database, partialSchema = _a.partialSchema;
    var schema = graphql_utils_1.createFullSchema(partialSchema);
    return new apollo_link_1.ApolloLink(function (operation, forward) {
        var isFirestore = apollo_utilities_1.hasDirectives(["firestore"], operation.query);
        if (!isFirestore) {
            return forward ? forward(operation) : null;
        }
        var query = operation.query, variables = operation.variables, operationName = operation.operationName;
        var context = { database: database };
        var rootValue = {};
        return new apollo_link_1.Observable(function (observer) {
            var result = graphql_1.execute(schema, query, rootValue, context, variables, operationName);
            result.then(function (data) {
                observer.next({ data: data });
                observer.complete();
            })
                .catch(function (err) {
                if (err.name === "AbortError") {
                    return;
                }
                if (err.result && err.result.errors) {
                    observer.next(err.result);
                }
                observer.error(err);
            });
        });
    });
}
exports.createFirestoreLink = createFirestoreLink;
