"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
function createFullSchema(partialSchema) {
    var typeMapping = {
        String: graphql_1.GraphQLString,
        Int: graphql_1.GraphQLInt,
        ID: graphql_1.GraphQLID,
        Boolean: graphql_1.GraphQLBoolean,
        Float: graphql_1.GraphQLFloat,
    };
    function createFieldType(type) {
        if (type.kind === "NonNullType") {
            return new graphql_1.GraphQLNonNull(createFieldType(type.type));
        }
        else if (type.kind === "ListType") {
            return new graphql_1.GraphQLList(createFieldType(type.type));
        }
        else {
            return typeMapping[type.name.value];
        }
    }
    function createObjectType(definition) {
        var typename = definition.name.value;
        if (!typeMapping[typename]) {
            typeMapping[typename] = new graphql_1.GraphQLObjectType({
                name: typename,
                fields: function () {
                    var fields = {};
                    for (var _i = 0, _a = definition.fields; _i < _a.length; _i++) {
                        var field = _a[_i];
                        fields[field.name.value] = { type: createFieldType(field.type) };
                    }
                    return fields;
                },
            });
        }
        return typeMapping[typename];
    }
    for (var _i = 0, _a = partialSchema.definitions; _i < _a.length; _i++) {
        var definition = _a[_i];
        if (definition.kind === "ObjectTypeDefinition") {
            createObjectType(definition);
        }
    }
    var queryType = new graphql_1.GraphQLObjectType({
        name: "Query",
        fields: function () {
            var fields = {};
            var _loop_1 = function (definition) {
                if (definition.kind === "ObjectTypeDefinition") {
                    var typename_1 = definition.name.value;
                    fields[typename_1.toLowerCase()] = {
                        type: typeMapping[typename_1],
                        args: {
                            id: { type: graphql_1.GraphQLID },
                        },
                        resolve: function (_, _a, context) {
                            var id = _a.id;
                            return context.database.collection(typename_1).doc(id).get();
                        },
                    };
                }
            };
            for (var _i = 0, _a = partialSchema.definitions; _i < _a.length; _i++) {
                var definition = _a[_i];
                _loop_1(definition);
            }
            return fields;
        },
    });
    var mutationType = new graphql_1.GraphQLObjectType({
        name: "Mutation",
        fields: function () {
            var fields = {};
            var _loop_2 = function (definition) {
                if (definition.kind === "ObjectTypeDefinition") {
                    var typename_2 = definition.name.value;
                    fields["create" + typename_2] = {
                        type: typeMapping[typename_2],
                        args: {
                            id: { type: graphql_1.GraphQLID },
                        },
                        resolve: function (_, args, context) {
                            return context.database.collection(typename_2).doc(args.id).set(args);
                        },
                    };
                }
            };
            for (var _i = 0, _a = partialSchema.definitions; _i < _a.length; _i++) {
                var definition = _a[_i];
                _loop_2(definition);
            }
            return fields;
        },
    });
    return new graphql_1.GraphQLSchema({
        query: queryType,
        mutation: mutationType,
    });
}
exports.createFullSchema = createFullSchema;
